/*
 *      Program do przegladania logow z ekg.
 *	Usuwanie czatow.
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CMD_DELCHAT_HH__
#define __CMD_DELCHAT_HH__


#include "command-base.h"
#include "texts.h"
#include "stuff.h"

using namespace EkgLog;

/**
 * @brief Komenda delchat
 */
class cmd_delchat_t : public command_base_t {
public:

	cmd_delchat_t() :  command_base_t("delchat", HELP_DELCHAT_SHORT,
				HELP_DELCHAT_LONG),
			_act_uin(0)
	{}

	void run(const std::vector<std::string> & argv)
	{
		if (_db->size() == 0)
			return;

		if (argv.size() > 1) {
			_info->print(DELCHAT_WRONG_ARGS, ALL_ERROR_FMT);
			return;
		}
		
		// tutaj powinien sprawdzac, czy lista czatow jest aktualna i
		// w razie odpowiedzi pozytywnej kasowac podswietlony
		if (argv.size() == 0)
			return;

		// zapewniamy poprawnosc wektora dopelnien - za chwile sie przyda
		completition_lvl2();

		unsigned int uin = _nicks->get_chosen_value();
		EkgLog::UinLog & log = _db->log(uin)->second;
		chat_iterator ci = log.begin();
		
		// znajdujemy w wektorze dopelnien szukany chat
		for (std::vector<std::string>::const_iterator i=_completition.begin();
				i!=_completition.end(); ++i, ++ci)
		{
			if (argv[0] == *i)
				break;
		}

		if (ci != log.end()) {
		//	log.delChat(ci);
		//	_info->print(format_date(ci->second.time()));
		//	_info->print(text_cast(ci->second.time()));
		}

		reload_chats();
		_info->print(DELCHAT_DELETED_P1+argv[0]+DELCHAT_DELETED_P2+_nicks->get_chosen_str()+
				DELCHAT_DELETED_P3+text_cast(_nicks->get_chosen_value()));

		_db_changed = true;

	} // run()

	virtual const std::vector<std::string> & completition_lvl2()
	{
		if (_act_uin == _nicks->get_chosen_value())
			return _completition;

		_completition.clear();
		_act_uin = _nicks->get_chosen_value();
		UinLog & log = _db->log(_act_uin)->second;

		for (chat_iterator i=log.begin(); i!=log.end(); ++i)
			_completition.push_back(format_date(i->second.time()));

		return _completition;
	}

private:
	int _act_uin;
};

#endif /* __CMD_DELCHAT_HH__ */
