/*
 *      Program do przegladania logow z ekg.
 *      Komenda przeszukiwania logow.
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "find.h"

#include <ctime>
#include <sys/time.h>

cmd_find_t::cmd_find_t() :  command_base_t("find", HELP_FIND_SHORT,
				HELP_FIND_LONG),
			_last_uin(0),_last_chat(0),
			_sr(0,0,0)
{
	_completition.push_back("-c");
	_completition.push_back("-t");
}

void cmd_find_t::run(const std::vector<std::string> & argv)
{
	using namespace EkgLog;

	if (argv.size() == 0) {
		_info->print(FIND_ERROR_NOARG, ALL_ERROR_FMT);
		_info->set_displayed(true);
		_main->set_displayed(false);
		return;
	}
	
	winlist_t::const_iterator wi = _nicks->begin();
	chat_iterator ci = _db->log(*wi)->second.begin();

	bool ignoreCase = true, thisChat = false;

	if (argv.size() > 3) {
		_info->print(FIND_TOO_MANY_ARGS, ALL_ERROR_FMT);
		return;
	}

	unsigned int size = argv.size();
	if (size)
		--size;

	for (unsigned int i=0; i<size; ++i) {
		if (argv[i] == "-c")
			ignoreCase = false;
		else if (argv[i] == "-t")
			thisChat = true;
		else {
			_info->print(FIND_WRONG_ARG_PART1 + argv[i] + FIND_WRONG_ARG_PART2,
					ALL_ERROR_FMT);
			_info->set_displayed(true);
			_main->set_displayed(false);
			_info->set_passive();
			return;
		}
	}

	// jesli jest co wyszukiwac
	bool continuation = _searched == argv.back();

	// wybrane opcje
	if (thisChat) {
		_searched = argv.back();
		_last_uin = _nicks->get_chosen_value();
		wi = _nicks->find(_last_uin);
		UinLog & log = _db->log(*wi)->second;
	
		_last_chat = _chats->get_chosen_value();
		if (_last_chat == 0) {
			ci = log.begin();
			_last_chat = ci->second.time();
		} else
			ci = log.chat(_last_chat);

		continuation = false;
	} else {
		if (!continuation) {
			_searched = argv.back();
			_sr = SearchResult(ci->second.begin(),0,0);
		} else {
			wi = _nicks->find(_last_uin);
			ci = _db->log(*wi)->second.chat(_last_chat);
		}
	}

	timeval start, finish;
	gettimeofday(&start, 0);

	for ( ; wi != _nicks->end(); ++wi) {
		UinLog & log = _db->log(*wi)->second;

		// ustaw ci tylko, jesli to nie jest ponowienie szukania
		if (_last_uin != (unsigned int)*wi || !continuation) {
			ci = log.begin();
			_sr = SearchResult(ci->second.begin(),0,0);
		}

		bool set_up = false;
		for ( ; ci!=log.end(); ++ci) {
			// ustaw _sr tylko wtedy, gdy w poprzednim przebiegu
			// petli nic nie znaleziono
			if (set_up)
				_sr = SearchResult(ci->second.begin(),0,0);

			try {
				_sr = ci->second.search(argv.back(),_sr, ignoreCase);
				
				if (_sr.size() != std::string::npos) {
					// znaleziono
					std::string found = FIND_FOUND_PART1 + _searched;
					_last_chat = ci->second.time();
					_last_uin  = *wi;

					gettimeofday(&finish, 0);
					found += FIND_FOUND_PART2 + log.nick();
					found += FIND_FOUND_PART3 + format_date(ci->second.time());
					found += FIND_FOUND_PART4 + text_cast(finish.tv_sec-start.tv_sec
						+ 0.001*((finish.tv_usec-start.tv_usec)/1000)) + " s";

					_info->print(found, ALL_INFO_FMT);

					_main->set_found(_sr, ci->second, log);
					_nicks->set_active_val(_last_uin);
					reload_chats();
					_chats->set_active_val(_last_chat);

					_main->set_passive();
					_main->set_displayed(true);
					_info->set_displayed(false);


					return;
				} else
					set_up = true;

			} catch (std::runtime_error & e) {
				_info->print(e.what(), ALL_ERROR_FMT);
				_searched.clear();
				return;
			}
		}
	}

	// jesli nie znaleziono ciagu trzeba wyczyscic info o wyszukiwaniu
	// poniewaz nastepne wyszukiwanie odnoszace sie do tego samego ciagu
	// zrobi segfaulta
	_searched.clear();
	_sr = SearchResult(0,0,0);
	gettimeofday(&finish, 0);
	_info->print(FIND_NOT_FOUND + text_cast(finish.tv_sec-start.tv_sec
		+ 0.001*((finish.tv_usec-start.tv_usec)/1000)) + " s", ALL_INFO_FMT);
	_info->set_displayed(true);
	_main->set_displayed(false);
	_info->set_passive();
} // run()
