/*
 *      Program do przegladania logow z ekg.
 *      Komenda zapisywania stanu bazy na dysk.
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CMD_SAVE_HH__
#define __CMD_SAVE_HH__

#include "command-base.h"
#include "texts.h"

/**
 * @brief Komenda save
 */
class cmd_save_t : public command_base_t {
public:

	cmd_save_t() :  command_base_t("save", HELP_SAVE_SHORT,
				HELP_SAVE_LONG)
	{
	}

	void run(const std::vector<std::string> & argv)
	{
		dBase::log_format_t fmt;
		const std::string & log = env.get_variable("log");
		if (log == "1")
			fmt = dBase::FILE;
		else if (log == "2")
			fmt = dBase::DIR;
		else if (log == "5")
			fmt = dBase::FILEGZ;
		else if (log == "6")
			fmt = dBase::DIRGZ;

		try {
			_db->save(env.get_variable("log_path"), fmt);
		} catch (std::runtime_error & e) {
			std::string tmp(SAVE_ERROR);
			_info->print(tmp + e.what(), ALL_ERROR_FMT);
			return;
		}
		_info->print(SAVE_SAVED, ALL_INFO_FMT);
		_db_changed = false;
	} // run()
};

#endif /* __CMD_SAVE_HH__ */
