/*
 *      Program do przegladania logow z ekg.
 *      Interfejs uzytkownika.
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __UI_HH__
#define __UI_HH__

#include "ui-basic.h"
#include "command-base.h"

#include "winlist.h"
#include "winmain.h"
#include "cmdline.h"
#include "ui-commands.h"
#include "environment.h"

#include "ekglog.hpp"
using namespace EkgLog;

/**
 * @brief Interfejs uzytkownika.
 *
 * Calosc interakcji z uzytkownikiem, zawiera w sobie wszystkie okna,
 * linie polecen, mape zarejestrowanych komend etc.
 */
class ui_t : public ui_basic_t {
	friend class command_base_t;
public:

	ui_t(dBase & db);
	~ui_t();

	/// glowna petla programu
	void loop();
	/// ustawia tekst informacyjny o bledach przetwarzania
	void set_dberror(const std::string & str);

private:

	/// odswieza zawartosc ekranu
	void refresh();
	/// Ustawia zawartosc list.
	void set_lists();

	/// wymiary okien
	unsigned int _list_width,_main_width,_main_height,
		     _nicks_height,_chats_height;
	/// okna ncurses dla obiektow-okien ui
	WINDOW * nc_main, *nc_nicks, *nc_chats, *nc_cmdline, *nc_status;
	/// dwa okna glowne - do wyswietlania rozmow i komunikatow
	winmain_t _main, _info;
	/// dwie listy wyboru - kontakty i rozmowy
	winlist_t _nicks, _chats;
	/// linia polecen
	cmdline_t _cmdline;
	/// pasek stanu
	winstatus_t _status;
	/// mapa z komendami interfejsu z pliku ui-commands.h
	std::map<std::string, command_base_t* > _cmds;
	/// ustawiajac to na true, konczy sie program
	bool _end_program;
	/// wskaznik na aktywne okno - jesli aktywna linia komend, to 0
	window_t * _active_window;

	/// referencja do bazy danych
	dBase & _db;

	/// informacje o bledach bazy
	bool _dberror;
	std::string _dberror_text;
};

#endif /* __UI_HH__ */
