/*
 *      Program do przegladania logow z ekg.
 *      Klasa podstawowa okna.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "window.h"

/* ze stuff.cc */
int color_pair(int fg, int bold, int bg);

window_t::window_t(const char *title,
	    int width, int height,
	    int margin_top, int margin_bottom, int margin_left, int margin_right,
	    int fg, int bold, int bg,
	    WINDOW *win,
	    bool cut_out)

	:_win(win)  ,
	 _width(width), _height(height),
	 _margin_top(margin_top), _margin_bottom(margin_bottom),
	 _margin_left(margin_left), _margin_right(margin_right),
	 _fg(fg), _bold(bold), _bg(bg),
	 _redraw(false), _act(false),_has_new(false),
	 _screen_start(0),
	 _cut_out(cut_out)
{
	if (title) {
		_title = title;
		++_margin_top;
	}
}


void window_t::print()
{
	if(!_redraw)
		return;

	werase(_win);

	if (_title.size()) {
		wattron(_win, color_pair(COLOR_WHITE, _act, COLOR_BLUE));
		wmove(_win, 0, 0);
		whline(_win, ' ', _width);

		if (_title.size() > hsize()) {
			wmove(_win, 0, 0);
			waddstr(_win, const_cast<char*>(_title.substr(0, hsize()).c_str()));
		} else {
			wmove(_win, 0, (_width-_title.size())/2);
			waddstr(_win, const_cast<char*>(_title.c_str()));
		}

		wattroff(_win, color_pair(COLOR_WHITE, _act, COLOR_BLUE));
	}

	unsigned int i=_screen_start;
	for( ; i<vsize()+_screen_start && i<_lines.size(); ++i) {
		wmove(_win, _margin_top+i-_screen_start, _margin_left);
		_lines[i].print(_win);
	}

	// wypisal ostatnia linie okna, a wiec na pewno nie ma juz nic nowego
	if(i == _lines.size())
		_has_new = false;

	_redraw = false;
	wnoutrefresh(_win);
}


void window_t::print(const std::string & str, const std::string & fmt, bool message)
{
	unsigned int i=0, fmt_len=str_hidden(fmt), hidden=fmt_len, lastspc=0, cut=0;
	std::string tmp = fmt;
	
	// Trzeba obsluzyc zalamanie linii podczas zmienionego koloru.
	line_t::tag_t tag(_fg, _bold, _bg, 0), def=tag;

	/* Przeszukuje string i wynajduje znaki specjalne -> zmiany kolorow
	 * oraz lamanie linii. Jesli jest info o zmianie koloru, zapamietuje
	 * ilosc znakow, ktorych sie nie wyswietla i kopiuje dalej.
	 * Zeby porownac dlugosc stringa (czy dodac jako nowa linie) sprawdza
	 * dlugosc tempa pomniejszona o znaki ukryte.
	 */
	while (i < str.size()) {
		switch(str[i]) {
			case '\n':
				// tutaj byc moze trzeba dac cut+1
				addline(tmp, cut+1);
				cut = 0;
				tmp = fmt;
				hidden = fmt_len;
				/* Jesli kolory roznia sie od domyslnych dla okna
				 * to znaczy, ze linia jest lamana w miejscu zmienionego
				 * koloru i ta informacje trzeba przeniesc do nastepnej linii. */
				if(!(tag == def)) {
					tmp += tag;
					hidden += 5;
				}
				break;

			case '\\':
				if (message) {
					tmp += '\\';
				} else if (i+1 < str.size()) {
					if (str[i+1] == 'c') {
						tag(str[i+2]-'0',str[i+3]-'0',str[i+4]-'0',0);
						tmp += str.substr(i, 5);
						i += 4;
						hidden += 5;

					} else if (str[i+1] == 'd') {
						tmp += str.substr(i, 2);
						i += 1;
						hidden += 2;
						tag(_fg,_bold,_bg,0);

					} else if (str[i+1] == '\\') {
						// dodajemy podwojnie, bo konstruktor linii
						// tez sprawdza te znaki
						tmp += "\\\\";
						hidden += 1;
						i += 1;
						++cut;
					}
				}

				break;
			case ' ':
				lastspc = tmp.size();
			default:
				tmp += str[i];
				break;
		}
		if (tmp.size()-hidden >= hsize()) {
			if(tmp[tmp.size()-1] != ' ' && lastspc) {
				i -= tmp.size()-lastspc-1;
				tmp.erase(lastspc+1, std::string::npos);
			}
			// tutaj chyba trzeba doliczyc do wycietych znakow spacje
			// w ktorej zlamano wiersz
			addline(tmp, cut);
			cut = 0;
			tmp = fmt;
			hidden = fmt_len;
			// na wypadek zlamania w kolorowanym tekscie
			if(!(tag == def)) {
				tmp += tag;
				hidden += 5;
			}
		}

		++i;
	}
	if (tmp.size()) {
		addline(tmp, cut);
	}

	_redraw = true;
	_has_new = true;
}


void window_t::clear()
{
	_lines.clear();
	_cut.clear();
	_screen_start = 0;
	_has_new = false;
	_redraw = true;
}


void window_t::addline(const std::string & str, unsigned int cut)
{
	_cut.push_back(cut);
	_lines.push_back(line_t(str.c_str(), _fg, _bold, _bg));
	if (_lines.size() > vsize() && (_screen_start == _lines.size()-vsize()-1))
		++_screen_start;
}


void window_t::moveup()
{
	if (_screen_start > 0)
		--_screen_start;

	_redraw = true;
}


void window_t::movedown()
{
	if (_screen_start+vsize() < _lines.size())
		++_screen_start;

	_redraw = true;
}


void window_t::movepgup()
{
	for(unsigned int i=0; i<vsize(); ++i)
		moveup();
}


void window_t::movepgdown()
{
	for(unsigned int i=0; i<vsize(); ++i)
		movedown();

}


void window_t::movehome()
{
	_screen_start = 0;
	_redraw = true;
}

void window_t::moveend()
{
	if (_lines.size() <= vsize())
		_screen_start = 0;
	else
		_screen_start = _lines.size()-vsize();

	_redraw = true;
}

