/*
 *      Program do przegladania logow z ekg.
 *      Glowne okno.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "winmain.h"
#include "texts.h"


winmain_t::winmain_t(const char *title, 
	    int width, int height,
	    int history_lenght,
	    WINDOW *win)
	: window_t(title, width, height, 0,0,0,0, 7,0,0, win, true),
	  _history_lenght(history_lenght),_on_screen(false),
	  _act_uin(0),_act_chat(0)
{
}

void winmain_t::print()
{
	if(!_on_screen)
		return;

	return window_t::print();
}

    
void winmain_t::add_chat(const EkgLog::Chat & chat, const EkgLog::UinLog & ulog)
{
	using namespace EkgLog;
	// ustaw czat na pierwszej linii i _has_new na true, jesli
	// chat.size() > vsize()
	// _has_new = true;
	clear();
	std::string tmp;
	std::string msg;

	msg_iterator i = chat.begin();
	for ( ; i!=chat.end(); ++i) {
		msg = i->msg();

		if (i->type() == MSG_RECV) {
			tmp = MSG_RECV_BEGIN;
			tmp.replace(tmp.find("{%nick}"), 7, ulog.nick());
			tmp.replace(tmp.find("{%uin}"), 6, text_cast(ulog.uin()));
			print(tmp);
			_msg_start.push_back(_lines.size());
			_msg_iter.push_back(i);
			print(msg, MSG_RECV_MIDDLE, true);
			print(MSG_RECV_END);
		} else {
			tmp = MSG_SENT_BEGIN;
			tmp.replace(tmp.find("{%nick}"), 7, env.nick());
			tmp.replace(tmp.find("{%uin}"), 6, text_cast(env.uin()));
			print(tmp);
			_msg_start.push_back(_lines.size());
			_msg_iter.push_back(i);
			print(msg, MSG_SENT_MIDDLE, true);
			print(MSG_SENT_END);
		}
	}

	_screen_start = 0;
	_has_new = true;
	if (_lines.size() > vsize())
		_has_new = true;
}


void winmain_t::set_found(	const EkgLog::SearchResult & sr, 
				const EkgLog::Chat & chat,
				const EkgLog::UinLog & ulog )
{
	if (chat.time() != _act_chat || ulog.uin() != _act_uin) {
		add_chat(chat, ulog);
		_act_uin = ulog.uin();
		_act_chat = chat.time();
	} else {
		// podmieniamy z powrotem pokolowane linie
		for (unsigned int i=0; i<_colored_line.size(); ++i) {
			_lines[_colored_no[i]] = _colored_line[i];
		}
		_colored_line.clear();
		_colored_no.clear();
	}

	std::vector<msg_iterator>::iterator i = find(_msg_iter.begin(), _msg_iter.end(),
					sr.message());
	if (i == _msg_iter.end())
		return;

	_screen_start = _msg_start[i-_msg_iter.begin()];
	unsigned int line=_screen_start, off=0, begin=sr.begin();

	// pierwsza rzecz to znalezienie poczatku znalezionego ciagu
	while (begin > 0) {
		// dlugosc tekstu to dlugosc linii minus ciag formatujacy
		unsigned int len = _lines[line].size() + _cut[line] - 2;

		if (len<= begin) {
			begin -= len;
			++line;
		} else {
			off = begin;
			begin = 0;
		}
	}

	_screen_start=line;

	// teraz trzeba pokolorowac znaleziony tekst
	// wszedzie jest line.size()-2, bo przed wlasciwym tekstem w kazdej
	// linii wystepuje ciag "| "
	unsigned int length = sr.length();
	_colored_line.push_back(_lines[line]);
	_colored_no.push_back(line);

	if (off + length > _lines[line].size()-2) {
		_lines[line].addtag(line_t::tag_t(0,0,6, _lines[line].size()-off-2),off+2);
		length -= _lines[line].size()-off-2;
	} else {
		_lines[line].addtag(line_t::tag_t(0,0,6, length),off+2);
		length = 0;
	}

	++line;
	while (length > 0) {
		_colored_line.push_back(_lines[line]);
		_colored_no.push_back(line);

		if (length > _lines[line].size()-2) {
			_lines[line].addtag(line_t::tag_t(0,0,6, _lines[line].size()-2), 2);
			length -= _lines[line].size()-2;
		} else {
			_lines[line].addtag(line_t::tag_t(0,0,6, length), 2);
			length = 0;
		}
		++line;
	}
}

void winmain_t::dump(std::ofstream & out)
{
	for (std::vector<line_t>::const_iterator i=_lines.begin(); i!=_lines.end(); ++i)
		out << i->str() << '\n';
}


void winmain_t::clear()
{
	window_t::clear();

	//_redraw  = false;
	_has_new = false;
	_history_lenght = 0; 
	_msg_start.clear();
	_msg_iter.clear();
	_colored_line.clear();
	_colored_no.clear();
	_act_chat = 0;
	_act_uin = 0;
}


void winmain_t::add_statuses(const EkgLog::UinLog & log)
{
	using namespace EkgLog;

	clear();
	for (const_status_iterator i=log.statusBegin(); i!=log.statusEnd(); ++i)
		print("\\c710\"\\d"+i->desc()+"\\c710\"\\d");

	_screen_start = 0;
	_has_new = true;
	if (_lines.size() > vsize())
		_has_new = true;
}
